@@ -4,7 +4,7 @@
 fprod <- function(x, ...) UseMethod("fprod") # , x
 
 fprod.default <- function(x, g = NULL, w = NULL, TRA = NULL, na.rm = .op[["na.rm"]], use.g.names = TRUE, ...) {
-  if(is.matrix(x) && !inherits(x, "matrix")) return(fprod.matrix(x, g, w, TRA, na.rm, use.g.names, ...))
+  # if(is.matrix(x) && !inherits(x, "matrix")) return(fprod.matrix(x, g, w, TRA, na.rm, use.g.names, ...))
   if(is.null(TRA)) {
     if(!missing(...)) unused_arg_action(match.call(), ...)
     if(is.null(g)) return(.Call(C_fprod,x,0L,0L,w,na.rm))
@@ -50,6 +50,9 @@ fprod.matrix <- function(x, g = NULL, w = NULL, TRA = NULL, na.rm = .op[["na.rm"
   TRAmC(x,.Call(C_fprodm,x,g[[1L]],g[[2L]],w,na.rm,FALSE),g[[2L]],TRA, ...)
 }
 
+fprod.zoo <- function(x, ...) if(is.matrix(x)) fprod.matrix(x, ...) else fprod.default(x, ...)
+fprod.units <- function(x, ...) if(is.matrix(x)) copyMostAttrib(fprod.matrix(x, ...), x) else fprod.default(x, ...)
+
 fprod.data.frame <- function(x, g = NULL, w = NULL, TRA = NULL, na.rm = .op[["na.rm"]], use.g.names = TRUE, drop = TRUE, ...) {
   if(is.null(TRA)) {
     if(!missing(...)) unused_arg_action(match.call(), ...)
@@ -77,7 +80,7 @@ fprod.data.frame <- function(x, g = NULL, w = NULL, TRA = NULL, na.rm = .op[["na
 fprod.list <- function(x, ...) fprod.data.frame(x, ...)
 
 fprod.grouped_df <- function(x, w = NULL, TRA = NULL, na.rm = .op[["na.rm"]], use.g.names = FALSE,
-                            keep.group_vars = TRUE, keep.w = TRUE, ...) {
+                            keep.group_vars = TRUE, keep.w = TRUE, stub = .op[["stub"]], ...) {
   g <- GRP.grouped_df(x, call = FALSE)
   if(is.null(g[[4L]])) keep.group_vars <- FALSE
   wsym <- substitute(w)
@@ -92,7 +95,7 @@ fprod.grouped_df <- function(x, w = NULL, TRA = NULL, na.rm = .op[["na.rm"]], us
       if(any(gn %in% wn)) stop("Weights coincide with grouping variables!")
       gn <- c(gn, wn)
       if(keep.w) {
-        if(nTRAl) prodw <- `names<-`(list(.Call(C_fprod,w,g[[1L]],g[[2L]],NULL,na.rm)), paste0("prod.", if(length(wsym) == 1L) wsym else deparse(wsym))) else if(keep.group_vars)
+        if(nTRAl) prodw <- `names<-`(list(.Call(C_fprod,w,g[[1L]],g[[2L]],NULL,na.rm)), do_stub(stub, if(length(wsym) == 1L) as.character(wsym) else deparse(wsym), "prod.")) else if(keep.group_vars)
           gn2 <- gn else prodw <- gn2 <- wn
       }
     }
