@@ -155,7 +155,7 @@ join <- function(x, y,
   }
 
   # Check for duplicate columns and suffix as needed
-  if(any(nm <- match(ynam[-iyon], xnam, nomatch = 0L))) {
+  if(any(nm <- match(ynam[-iyon], xnam, nomatch = 0L)) && switch(how, semi = FALSE, anti = FALSE, TRUE)) {
     nnm <- nm != 0L
     nam <- xnam[nm[nnm]]
     if(is.character(drop.dup.cols) || drop.dup.cols) {
@@ -293,8 +293,10 @@ join <- function(x, y,
                  anti = structure(alloc(1L, fnrow(res)), levels = x_name, class = c("factor", "na.included")))
     attr(mc, "on.cols") <- `names<-`(list(xon, `names<-`(on, NULL)), c(x_name, y_name))
     mc_name <- if(is.character(column)) column else ".join"
-    if(keep.col.order == 1L) res[[mc_name]] <- mc
-    else res <- c(res[ixon], `names<-`(list(mc), mc_name), res[-ixon])
+    if(keep.col.order == 1L) res[[mc_name]] <- mc else {
+      if(keep.col.order == 2L) ixon <- seq_along(ixon)
+      res <- c(res[ixon], `names<-`(list(mc), mc_name), res[-ixon])
+    }
   } else if(!keep.col.order) res <- c(res[ixon], res[-ixon])
 
   # Final steps
