@@ -131,7 +131,7 @@ SEXP dupVecIndex(SEXP x) {
     const double *restrict px = REAL(x);
     union uno tpv;
     for (int i = 0; i != n; ++i) {
-      tpv.d = px[i]+0; // R_IsNA(px[i]) ? NA_REAL : (R_IsNaN(px[i]) ? R_NaN : px[i]);
+      tpv.d = px[i]; // R_IsNA(px[i]) ? NA_REAL : (R_IsNaN(px[i]) ? R_NaN : px[i]);
       id = HASH(tpv.u[0] + tpv.u[1], K);
       while(h[id]) {
         if(REQUAL(px[h[id]-1], px[i])) {
@@ -157,9 +157,9 @@ SEXP dupVecIndex(SEXP x) {
       } else if (C_IsNaN(tmp)) {
         tmp.r = tmp.i = R_NaN;
       }
-      tpv.d = tmp.r+0;
+      tpv.d = tmp.r;
       u = tpv.u[0] ^ tpv.u[1];
-      tpv.d = tmp.i+0;
+      tpv.d = tmp.i;
       u ^= tpv.u[0] ^ tpv.u[1];
       id = HASH(u, K);
       while(h[id]) {
@@ -261,7 +261,7 @@ SEXP dupVecIndexKeepNA(SEXP x) {
             goto ibl;
           } // else, we move forward to the next slot, until we find an empty one... We need to keep checking against the values,
           // because if we found the same value before, we would also have put it in another slot after the initial one with the same hash value.
-          if(++id >= nu) id = 0; // ++iid; iid %= nu; // # nocov
+          if(++iid >= nu) iid = 0; // ++iid; iid %= nu; // # nocov
         } // We put the index into the empty slot.
         h[iid] = i + 1; // need + 1 because for zero the while loop gives false..
         pans_i[i] = ++g; // h[id];
@@ -295,7 +295,7 @@ SEXP dupVecIndexKeepNA(SEXP x) {
         pans_i[i] = NA_INTEGER;
         continue;
       }
-      tpv.d = px[i]+0;
+      tpv.d = px[i];
       id = HASH(tpv.u[0] + tpv.u[1], K);
       while(h[id]) {
         if(REQUAL(px[h[id]-1], px[i])) {
@@ -320,9 +320,9 @@ SEXP dupVecIndexKeepNA(SEXP x) {
         pans_i[i] = NA_INTEGER;
         continue;
       }
-      tpv.d = tmp.r+0;
+      tpv.d = tmp.r;
       u = tpv.u[0] ^ tpv.u[1];
-      tpv.d = tmp.i+0;
+      tpv.d = tmp.i;
       u ^= tpv.u[0] ^ tpv.u[1];
       id = HASH(u, K);
       while(h[id]) {
@@ -473,7 +473,7 @@ SEXP dupVecIndexTwoVectors(SEXP x, SEXP y) {
           union uno tpx, tpy;
           // fill hash table with indices of 'table'
           for (int i = 0; i != n; ++i) {
-            tpx.d = px[i]+0; tpy.d = py[i]+0;
+            tpx.d = px[i]; tpy.d = py[i];
             id = HASH((64988430769U * (tpx.u[0] + tpx.u[1])) + tpy.u[0] + tpy.u[1], K); // Best combination it seems
             while(h[id]) {
               hid = h[id]-1;
@@ -497,7 +497,7 @@ SEXP dupVecIndexTwoVectors(SEXP x, SEXP y) {
         const double *restrict pr = REAL_RO(tx == REALSXP ? x : y);
         union uno tpv;
         for (int i = 0; i != n; ++i) {
-          tpv.d = pr[i]+0;
+          tpv.d = pr[i];
           id = HASH((64988430769U * pi[i]) + tpv.u[0] + tpv.u[1], K); // Best combination it seems
           while(h[id]) {
             hid = h[id]-1;
@@ -517,7 +517,7 @@ SEXP dupVecIndexTwoVectors(SEXP x, SEXP y) {
         const double *restrict pr = REAL_RO(tx == REALSXP ? x : y);
         union uno tpv;
         for (int i = 0; i != n; ++i) {
-          tpv.d = pr[i]+0;
+          tpv.d = pr[i];
           id = HASH((tpv.u[0] + tpv.u[1]) * ((uintptr_t)ps[i] & 0xffffffff), K); // Best combination it seems
           while(h[id]) {
             hid = h[id]-1;
@@ -655,7 +655,7 @@ int dupVecSecond(int *restrict pidx, int *restrict pans_i, SEXP x, const int n,
     const unsigned int mult = (M-1) / ng; // -1 because C is zero indexed
     union uno tpv;
     for (int i = 0; i != n; ++i) {
-      tpv.d = px[i]+0; // R_IsNA(px[i]) ? NA_REAL : (R_IsNaN(px[i]) ? R_NaN :px[i]);
+      tpv.d = px[i]; // R_IsNA(px[i]) ? NA_REAL : (R_IsNaN(px[i]) ? R_NaN :px[i]);
       id = (pidx[i]*mult) ^ HASH(tpv.u[0] + tpv.u[1], K); // HASH((tpv.u[0] + tpv.u[1]) ^ pidx[i], K) + pidx[i]; // Note: This is much faster than just adding pidx[i] to the hash value...
       while(h[id]) { // Problem: This value might be seen before, but not in combination with that pidx value...
         hid = h[id]-1; // The issue here is that REQUAL(px[hid], px[i]) could be true but pidx[hid] == pidx[i] fails, although the same combination of px and pidx could be seen earlier before...
@@ -683,9 +683,9 @@ int dupVecSecond(int *restrict pidx, int *restrict pans_i, SEXP x, const int n,
       } else if (C_IsNaN(tmp)) {
         tmp.r = tmp.i = R_NaN;
       }
-      tpv.d = tmp.r+0;
+      tpv.d = tmp.r;
       u = tpv.u[0] ^ tpv.u[1];
-      tpv.d = tmp.i+0;
+      tpv.d = tmp.i;
       u ^= tpv.u[0] ^ tpv.u[1];
       id = (pidx[i]*mult) ^ HASH(u, K);
       while(h[id]) {
@@ -765,30 +765,36 @@ SEXP groupVec(SEXP X, SEXP starts, SEXP sizes) {
       SEXP gs, st, starts_sym = install("starts"), sizes_sym = install("group.sizes");
       setAttrib(res, starts_sym, st = allocVector(INTSXP, ng));
       setAttrib(res, sizes_sym, gs = allocVector(INTSXP, ng));
-      int *pgs = INTEGER(gs), *pst = INTEGER(st);
-      memset(pgs, 0, sizeof(int) * ng); --pgs;
-      memset(pst, 0, sizeof(int) * ng); --pst;
-      for(int i = 0; i != n; ++i) {
-        ++pgs[pres[i]];
-        if(pst[pres[i]] == 0) pst[pres[i]] = i + 1;
+      if(ng > 0) {
+        int *pgs = INTEGER(gs), *pst = INTEGER(st);
+        memset(pgs, 0, sizeof(int) * ng); --pgs;
+        memset(pst, 0, sizeof(int) * ng); --pst;
+        for(int i = 0; i != n; ++i) {
+          ++pgs[pres[i]];
+          if(pst[pres[i]] == 0) pst[pres[i]] = i + 1;
+        }
       }
     } else if(start) {
       SEXP st, starts_sym = install("starts");
       setAttrib(res, starts_sym, st = allocVector(INTSXP, ng));
-      int *pst = INTEGER(st), k = 0;
-      memset(pst, 0, sizeof(int) * ng); --pst;
-      for(int i = 0; i != n; ++i) {
-        if(pst[pres[i]] == 0) {
-          pst[pres[i]] = i + 1;
-          if(++k == ng) break;
+      if(ng > 0) {
+        int *pst = INTEGER(st), k = 0;
+        memset(pst, 0, sizeof(int) * ng); --pst;
+        for(int i = 0; i != n; ++i) {
+          if(pst[pres[i]] == 0) {
+            pst[pres[i]] = i + 1;
+            if(++k == ng) break;
+          }
         }
       }
     } else {
       SEXP gs, sizes_sym = install("group.sizes");
       setAttrib(res, sizes_sym, gs = allocVector(INTSXP, ng));
-      int *pgs = INTEGER(gs);
-      memset(pgs, 0, sizeof(int) * ng); --pgs;
-      for(int i = 0; i != n; ++i) ++pgs[pres[i]];
+      if(ng > 0) {
+        int *pgs = INTEGER(gs);
+        memset(pgs, 0, sizeof(int) * ng); --pgs;
+        for(int i = 0; i != n; ++i) ++pgs[pres[i]];
+      }
     }
   }
   UNPROTECT(nprotect);
@@ -807,20 +813,22 @@ SEXP groupAtVec(SEXP X, SEXP starts, SEXP naincl) {
   SEXP st, sym_ng = install("N.groups"), starts_sym = install("starts");
   int ng = asInteger(getAttrib(idx, sym_ng)), n = length(idx), *pidx = INTEGER(idx);
   setAttrib(idx, starts_sym, st = allocVector(INTSXP, ng));
-  int *pst = INTEGER(st), k = 0;
-  memset(pst, 0, sizeof(int) * ng); --pst;
-  if(nain) {
-    for(int i = 0; i != n; ++i) {
-      if(pst[pidx[i]] == 0) {
-        pst[pidx[i]] = i + 1;
-        if(++k == ng) break;
+  if(ng > 0) {
+    int *pst = INTEGER(st), k = 0;
+    memset(pst, 0, sizeof(int) * ng); --pst;
+    if(nain) {
+      for(int i = 0; i != n; ++i) {
+        if(pst[pidx[i]] == 0) {
+          pst[pidx[i]] = i + 1;
+          if(++k == ng) break;
+        }
       }
-    }
-  } else {
-    for(int i = 0; i != n; ++i) {
-      if(pidx[i] != NA_INTEGER && pst[pidx[i]] == 0) {
-        pst[pidx[i]] = i + 1;
-        if(++k == ng) break;
+    } else {
+      for(int i = 0; i != n; ++i) {
+        if(pidx[i] != NA_INTEGER && pst[pidx[i]] == 0) {
+          pst[pidx[i]] = i + 1;
+          if(++k == ng) break;
+        }
       }
     }
   }
@@ -930,7 +938,7 @@ SEXP funiqueC(SEXP x) {
     const double *restrict px = REAL(x);
     union uno tpv;
     for (int i = 0; i != n; ++i) {
-      tpv.d = px[i]+0;
+      tpv.d = px[i];
       id = HASH(tpv.u[0] + tpv.u[1], K);
       while(h[id]) {
         if(REQUAL(px[h[id]-1], px[i])) goto rbl;
@@ -961,9 +969,9 @@ SEXP funiqueC(SEXP x) {
       } else if (C_IsNaN(tmp)) {
         tmp.r = tmp.i = R_NaN;
       }
-      tpv.d = tmp.r+0;
+      tpv.d = tmp.r;
       u = tpv.u[0] ^ tpv.u[1];
-      tpv.d = tmp.i+0;
+      tpv.d = tmp.i;
       u ^= tpv.u[0] ^ tpv.u[1];
       id = HASH(u, K);
       while(h[id]) {
