@@ -1,6 +1,6 @@
 context("Attribute Handling")
 
-if(!is.null(attributes(identical(FALSE, TRUE)))) stop("OECD label issue")
+
 
 v <- wlddev$PCGDP
 date <- wlddev$date
@@ -8,7 +8,7 @@ fac <- wlddev$region
 g1 <- GRP(wlddev$country)
 m <- qM(mtcars)
 gmtc <- fgroup_by(mtcars, cyl, vs, am)
-gm <- qM(gmtc, TRUE)
+gm <- qM(gmtc, keep.attr = TRUE)
 g2 <- GRP(mtcars, ~ cyl + vs + am)
 
 # gDTmtc <- fgroup_by(qDT(mtcars), cyl, vs, am)
@@ -19,6 +19,8 @@ f2 <- sample.int(5, nrow(EuStockMarkets), replace = TRUE)
 # numFUN <- setdiff(.FAST_STAT_FUN, c("fnth", "fmode", "ffirst", "flast", "fmin", "fmax"))
 countFUN <- c("fnobs", "fndistinct")
 
+F <- getNamespace("collapse")$F
+
 test_that("statistical functions handle attributes properly", {
 
   for(i in setdiff(.FAST_STAT_FUN, "fnth")) {
