@@ -467,6 +467,13 @@ test_that("fmedian ties handled properly with weights", {
 
 test_that("Misc bugs", {
   expect_visible(qF(c(4L, 1L, NA), sort = FALSE))
+  expect_equal(fmatch(factor(NA, exclude = NULL), NA), 1L) # #675
+  expect_equal(fmatch(factor(NA), NA), 1L)
+  expect_visible(qsu(mtcars$mpg, mtcars$cyl, mtcars$vs, mtcars$wt))
+  expect_visible(qsu(mtcars$mpg, mtcars$cyl, mtcars$vs, mtcars$wt, higher = TRUE))
+  expect_visible(qsu(mtcars$mpg, mtcars$cyl, mtcars$vs, mtcars$wt, array = FALSE))
+  expect_visible(qsu(mtcars$mpg, mtcars$cyl, mtcars$vs, mtcars$wt, higher = TRUE, array = FALSE))
 })
 
+
 options(warn = 1)
